/*
 * Copyright (C) 2007-2011 Linutop SAS <http://www.linutop.com/>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>

#include "libappstarter/as-utils.h"

static gchar *unclutter[] = {
    "unclutter",
    "-root",
    "-idle",
    "1",
    NULL
};

static void
run_cmd (const gchar *cmd, GPid unclutter_pid)
{
    int r;

    /* loop to restart the application if it crashed */
    while (1)
    {
        r = system (cmd);
        if (WIFEXITED (r) && (WEXITSTATUS (r) == 0))
            break;
    }

    if (unclutter_pid)
        kill (unclutter_pid, SIGTERM);
}

int
main (int argc, char *argv[])
{
    pid_t child;
    XfceRc *rc;
    int do_fork = 1;
    const gchar *cmd;
    gboolean hidecursor;
    GPid unclutter_pid = 0;

    rc = as_rc_open (NULL, TRUE);
    if (!rc)
        return 1;

    xfce_rc_set_group (rc, "default");
    cmd = xfce_rc_read_entry (rc, "command", "true");
    hidecursor = xfce_rc_read_bool_entry (rc, "hidecursor", TRUE);
    xfce_rc_close (rc);

    if (hidecursor)
        g_spawn_async (NULL, unclutter, NULL, G_SPAWN_SEARCH_PATH,
                       NULL, NULL, &unclutter_pid, NULL);

    if ((argc > 1) && (strcmp (argv[1], "--nofork") == 0))
        run_cmd (cmd, unclutter_pid);

    else {
        if ((child = fork()) < 0)
            return 1;
        else if (child == 0)
            run_cmd (cmd, unclutter_pid);
        else
            exit(0);
    }

    return 0;
}
