/*
 * Copyright (C) 2011 Linutop SAS <http://www.linutop.com/>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib/gprintf.h>
#include <stdlib.h>

#include "libappstarter/as.h"

#define PLUGIN "remote"

enum {
    PROTO_RDP = 0,
    PROTO_VNC
};

static gchar *cmd = NULL;
static gchar *conffile;

static guint proto;
static gboolean fullscreen;
static gchar *host = NULL;
static gchar *user = NULL;
static gchar *passwd = NULL;
static gchar *extra = NULL;

static GtkWidget *vnc_btn, *rdp_btn;
static GtkWidget *fullscreen_chk;
static GtkWidget *host_entry, *user_entry, *passwd_entry;
static GtkWidget *extra_entry;

static void
read_config ()
{
    XfceRc *rc;

    rc = as_rc_open (conffile, TRUE);
    if (!rc)
        return;

    xfce_rc_set_group (rc, PLUGIN);
    proto = xfce_rc_read_int_entry (rc, "proto", PROTO_RDP);
    host = g_strdup (xfce_rc_read_entry (rc, "host", ""));
    user = g_strdup (xfce_rc_read_entry (rc, "user", ""));
    passwd = g_strdup (xfce_rc_read_entry (rc, "passwd", ""));
    extra = g_strdup (xfce_rc_read_entry (rc, "extra", ""));
    fullscreen = xfce_rc_read_bool_entry (rc, "fullscreen", TRUE);
    xfce_rc_close (rc);
}

static void
write_config (GtkWidget *widget, gpointer data)
{
    XfceRc *rc;

    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (vnc_btn)))
    {
        proto = PROTO_VNC;
        gtk_widget_set_sensitive (passwd_entry, FALSE);
    }
    else
    {
        proto = PROTO_RDP;
        gtk_widget_set_sensitive (passwd_entry, TRUE);
    }

    rc = as_rc_open (conffile, FALSE);
    if (!rc)
        return;

    if (host)
        g_free (host);
    if (user)
        g_free (user);
    if (passwd)
        g_free (passwd);
    if (extra)
        g_free (extra);

    xfce_rc_set_group (rc, PLUGIN);
    xfce_rc_write_int_entry (rc, "proto", proto);
    host = g_strdup (gtk_entry_get_text (GTK_ENTRY (host_entry)));
    xfce_rc_write_entry (rc, "host", host);
    user = g_strdup (gtk_entry_get_text (GTK_ENTRY (user_entry)));
    xfce_rc_write_entry (rc, "user", user);
    passwd = g_strdup (gtk_entry_get_text (GTK_ENTRY (passwd_entry)));
    xfce_rc_write_entry (rc, "passwd", passwd);
    extra = g_strdup (gtk_entry_get_text (GTK_ENTRY (extra_entry)));
    xfce_rc_write_entry (rc, "extra", extra);
    fullscreen = gtk_toggle_button_get_active (
                            GTK_TOGGLE_BUTTON (fullscreen_chk));
    xfce_rc_write_bool_entry (rc, "fullscreen", fullscreen);
    xfce_rc_close (rc);
}

GtkWidget *
get_config_widget (gchar *file)
{
    GtkWidget *vbox, *hbox, *vbox2;
    GtkWidget *label;
    GtkSizeGroup *sg;

    conffile = g_strdup (file);
    read_config ();

    vbox = gtk_vbox_new (FALSE, 2);
    sg = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);

    label = gtk_label_new (_("This plugin will start a connection to a remote "
                             "system.\n"));
    gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
    gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 2);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 2);
    label = gtk_label_new (_("Protocol:"));
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
    gtk_size_group_add_widget (sg, label);
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2);

    vbox2 = gtk_vbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (hbox), vbox2, FALSE, FALSE, 2);

    vnc_btn = gtk_radio_button_new_with_label (NULL, _("VNC"));
    gtk_box_pack_start (GTK_BOX (vbox2), vnc_btn, FALSE, FALSE, 0);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (vnc_btn),
        (proto == PROTO_VNC) ? TRUE : FALSE);

    rdp_btn = gtk_radio_button_new_with_label_from_widget (
                GTK_RADIO_BUTTON (vnc_btn), _("RDP"));
    gtk_box_pack_start (GTK_BOX (vbox2), rdp_btn, FALSE, FALSE, 0);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rdp_btn),
        (proto == PROTO_RDP) ? TRUE : FALSE);

    g_signal_connect (G_OBJECT (vnc_btn), "toggled",
        G_CALLBACK (write_config), NULL);
    g_signal_connect (G_OBJECT (rdp_btn), "toggled",
        G_CALLBACK (write_config), NULL);

    fullscreen_chk = gtk_check_button_new_with_label (_("Fullscreen"));
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (fullscreen_chk), fullscreen);
    gtk_box_pack_start (GTK_BOX (vbox), fullscreen_chk, FALSE, FALSE, 2);
    g_signal_connect (G_OBJECT (fullscreen_chk), "toggled",
                      G_CALLBACK (write_config), NULL);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 2);
    label = gtk_label_new (_("Host:"));
    gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
    gtk_size_group_add_widget (sg, label);
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2);

    host_entry = gtk_entry_new ();
    gtk_entry_set_text (GTK_ENTRY (host_entry), host);
    gtk_box_pack_start (GTK_BOX (hbox), host_entry, FALSE, FALSE, 2);
    g_signal_connect (G_OBJECT (host_entry), "changed",
                      G_CALLBACK (write_config), NULL);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 2);
    label = gtk_label_new (_("User:"));
    gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
    gtk_size_group_add_widget (sg, label);
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2);

    user_entry = gtk_entry_new ();
    gtk_entry_set_text (GTK_ENTRY (user_entry), user);
    gtk_box_pack_start (GTK_BOX (hbox), user_entry, FALSE, FALSE, 2);
    g_signal_connect (G_OBJECT (user_entry), "changed",
                      G_CALLBACK (write_config), NULL);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 2);
    label = gtk_label_new (_("Password:"));
    gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
    gtk_size_group_add_widget (sg, label);
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2);

    passwd_entry = gtk_entry_new ();
    gtk_entry_set_text (GTK_ENTRY (passwd_entry), passwd);
    gtk_entry_set_visibility (GTK_ENTRY (passwd_entry), FALSE);
    gtk_box_pack_start (GTK_BOX (hbox), passwd_entry, FALSE, FALSE, 2);
    if (proto == PROTO_VNC)
        gtk_widget_set_sensitive (passwd_entry, FALSE);
    g_signal_connect (G_OBJECT (passwd_entry), "changed",
                      G_CALLBACK (write_config), NULL);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 2);
    label = gtk_label_new (_("Extra options:"));
    gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
    gtk_size_group_add_widget (sg, label);
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2);

    extra_entry = gtk_entry_new ();
    gtk_entry_set_text (GTK_ENTRY (extra_entry), extra);
    gtk_box_pack_start (GTK_BOX (hbox), extra_entry, FALSE, FALSE, 2);
    g_signal_connect (G_OBJECT (extra_entry), "changed",
                      G_CALLBACK (write_config), NULL);

    gtk_widget_show_all (vbox);

    return vbox;
}

gchar *
get_cmd()
{
    gboolean with_vnc;
    gchar tmp[2048];

    with_vnc = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (vnc_btn));

    if (with_vnc)
    {
        strcpy (tmp, "vncviewer");
        if (fullscreen)
        {
            g_free (cmd);
            cmd = g_strdup (tmp);
            snprintf (tmp, 2047, "%s -fullscreen", cmd);
        }
        if (user && (strcmp (user, "") != 0))
        {
            g_free (cmd);
            cmd = g_strdup (tmp);
            snprintf (tmp, 2047, "%s -user %s", cmd, user);
        }
        if (extra && (strcmp (extra, "") != 0))
        {
            g_free (cmd);
            cmd = g_strdup (tmp);
            snprintf (tmp, 2047, "%s %s", cmd, extra);
        }
        if (host && (strcmp (host, "") != 0)) /* must come last */
        {
            g_free (cmd);
            cmd = g_strdup (tmp);
            snprintf (tmp, 2047, "%s %s", cmd, host);
        }
    }
    else
    {
        strcpy (tmp, "rdesktop");
        if (fullscreen)
        {
            g_free (cmd);
            cmd = g_strdup (tmp);
            snprintf (tmp, 2047, "%s -f", cmd);
        }
        if (user && (strcmp (user, "") != 0))
        {
            g_free (cmd);
            cmd = g_strdup (tmp);
            snprintf (tmp, 2047, "%s -u %s", cmd, user);
        }
        if (passwd && (strcmp (passwd, "") != 0))
        {
            g_free (cmd);
            cmd = g_strdup (tmp);
            snprintf (tmp, 2047, "%s -p %s", cmd, passwd);
        }
        if (extra && (strcmp (extra, "") != 0))
        {
            g_free (cmd);
            cmd = g_strdup (tmp);
            snprintf (tmp, 2047, "%s %s", cmd, extra);
        }
        if (host && (strcmp (host, "") != 0)) /* must come last */
        {
            g_free (cmd);
            cmd = g_strdup (tmp);
            snprintf (tmp, 2047, "%s %s", cmd, host);
        }
    }

    g_free (cmd);
    cmd = g_strdup (tmp);

    return cmd;
}

void
free_all()
{
    if (host)
        g_free (host);
    if (user)
        g_free (user);
    if (passwd)
        g_free (passwd);
    if (extra)
        g_free (extra);
    g_free (conffile);
    g_free (cmd);
}
